/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.co.sharp.android.SampleStereo3DLCD_Kube;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.nio.ShortBuffer;
import java.util.Iterator;
import java.util.ArrayList;

import javax.microedition.khronos.opengles.GL10;

import android.opengl.GLU;

public class GLWorld {

    public void addShape(GLShape shape) {
        mShapeList.add(shape);
        mIndexCount += shape.getIndexCount();
    }
    
    public void generate() {        
        ByteBuffer bb = ByteBuffer.allocateDirect(mVertexList.size()*4*4);
        bb.order(ByteOrder.nativeOrder());
        mColorBuffer = bb.asIntBuffer();

        bb = ByteBuffer.allocateDirect(mVertexList.size()*4*3);
        bb.order(ByteOrder.nativeOrder());
        mVertexBuffer = bb.asIntBuffer();

        bb = ByteBuffer.allocateDirect(mIndexCount*2);
        bb.order(ByteOrder.nativeOrder());
        mIndexBuffer = bb.asShortBuffer();

        Iterator<GLVertex> iter2 = mVertexList.iterator();
        while (iter2.hasNext()) {
            GLVertex vertex = iter2.next();
            vertex.put(mVertexBuffer, mColorBuffer);
        }

        Iterator<GLShape> iter3 = mShapeList.iterator();
        while (iter3.hasNext()) {
            GLShape shape = iter3.next();
            shape.putIndices(mIndexBuffer);
        }
    }
    
    public GLVertex addVertex(float x, float y, float z) {
        GLVertex vertex = new GLVertex(x, y, z, mVertexList.size());
        mVertexList.add(vertex);
        return vertex;
    }
    
    public void transformVertex(GLVertex vertex, M4 transform) {
        vertex.update(mVertexBuffer, transform);
    }

    public void draw(GL10 gl)
    {
        mColorBuffer.position(0);
        mVertexBuffer.position(0);
        mIndexBuffer.position(0);

        /*
         *  SidebySide(Half)̍ڗp`s
         */
        if (multi)
            gl.glViewport(0, 0, w/2, h);
        else
            gl.glViewport(0, 0, w, h);
        
        float k = 0.5f * (zNear/D);
        float f = P * k * depthF;
        float W = w * k;
        float H = h * k;
           
        gl.glMatrixMode(GL10.GL_PROJECTION);
        gl.glLoadIdentity();
           
        gl.glFrustumf(-W - f, W - f, -H, H, zNear, zFar);
        
        gl.glMatrixMode(GL10.GL_MODELVIEW);
        gl.glLoadIdentity();
        
        GLU.gluLookAt(gl, 0.0f, 0.0f, defaultEyeZ,
                        0.0f, 0.0f, 0.0f,
                        0.0f, 1.0f, 0.0f);

        gl.glScalef(0.5f, 0.5f, 0.5f);
        gl.glRotatef(mAngle,        0, 1, 0);
        gl.glRotatef(mAngle*0.25f,  1, 0, 0);

        gl.glColor4f(0.7f, 0.7f, 0.7f, 1.0f);
        gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
        gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
        gl.glEnable(GL10.GL_CULL_FACE);
        gl.glShadeModel(GL10.GL_SMOOTH);
        gl.glEnable(GL10.GL_DEPTH_TEST);

        gl.glFrontFace(GL10.GL_CW);
        gl.glShadeModel(GL10.GL_FLAT);
        gl.glVertexPointer(3, GL10.GL_FIXED, 0, mVertexBuffer);
        gl.glColorPointer(4, GL10.GL_FIXED, 0, mColorBuffer);
        gl.glDrawElements(GL10.GL_TRIANGLES, mIndexCount, GL10.GL_UNSIGNED_SHORT, mIndexBuffer);

        /*
         *  SidebySide(Half)̉Eڗp`s
         */
        if (multi) {
            gl.glViewport(w/2, 0, w/2, h);
        
            gl.glMatrixMode(GL10.GL_PROJECTION);
            gl.glLoadIdentity();
           
            gl.glFrustumf(-W + f, W + f, -H, H, zNear, zFar);
        
            gl.glMatrixMode(GL10.GL_MODELVIEW);
            gl.glLoadIdentity();
        
            GLU.gluLookAt(gl, 0.0f, 0.0f, defaultEyeZ,
                        0.0f, 0.0f, 0.0f,
                        0.0f, 1.0f, 0.0f);

            gl.glScalef(0.5f, 0.5f, 0.5f);
            gl.glRotatef(mAngle,        0, 1, 0);
            gl.glRotatef(mAngle*0.25f,  1, 0, 0);

            gl.glColor4f(0.7f, 0.7f, 0.7f, 1.0f);
            gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
            gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
            gl.glEnable(GL10.GL_CULL_FACE);
            gl.glShadeModel(GL10.GL_SMOOTH);
            gl.glEnable(GL10.GL_DEPTH_TEST);

            gl.glFrontFace(GL10.GL_CW);
            gl.glShadeModel(GL10.GL_FLAT);
            gl.glVertexPointer(3, GL10.GL_FIXED, 0, mVertexBuffer);
            gl.glColorPointer(4, GL10.GL_FIXED, 0, mColorBuffer);
            gl.glDrawElements(GL10.GL_TRIANGLES, mIndexCount, GL10.GL_UNSIGNED_SHORT, mIndexBuffer);
        }
        
        count++;
    
    }
   
    static public float toFloat(int x) {
        return x/65536.0f;
    }

    private ArrayList<GLShape>    mShapeList = new ArrayList<GLShape>();    
    private ArrayList<GLVertex>    mVertexList = new ArrayList<GLVertex>();
    
    private int mIndexCount = 0;

    private IntBuffer   mVertexBuffer;
    private IntBuffer   mColorBuffer;
    private ShortBuffer mIndexBuffer;
    public float mAngle;
    int count = 0;
    public int w = 0;
    public int h = 0;
    public float eyeX = 0.01f;
    public float eyeY = 0.0f;
    public float eyeZ = 0.1f;
    public float defaultEyeZ = -6.0f;
    public boolean running = true;
    
    public float zNear = 0.1f;
    public float zFar = 50.0f;
    public float P = 15.0f;
    public float D = 2880.0f / 4;
    
    boolean multi = true;
    float depthF = 1.0f;
}
